// globaldefines.h  // ********************************************************
//
// user interface related routines for the project "USB-MidiCom",
// simple data converter for STM32F042F6 (20 pin TSSOP package)
// based on USB MIDISTREAM and CDC-ABSTRACT CONTROL MODEL
//
// Author and (c)2016-21 Wolfgang Schemmert. The copyright claim and
// the GPL-3 Licence is restricted to this file and the depending files
// "main.c", "main.h", "userinterface.c" and "usb-fs-042.c".
// Other depending material is published and copyrighted
// by STMicroelectronics and Segger GmbH/Rowley Associates Ltd
//
// Contact: www.midi-and-more.de
// A detailled hardware construction and operation manual
// is provided at this website
//
// ****************************************************************************
//      This program is free software: you can redistribute it and/or modify
//      it under the terms of the GNU General Public License as published by
//      the Free Software Foundation, either version 3 of the License, or
//      (at your option) any later version.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//      Licence: GPL-3
//
// ****************************************************************************

#ifndef GLOBALDEFINES
#define GLOBALDEFINES

// Version changes:
//REVNUM 2:	factoryReset() (opcode '!') added
//REVNUM 3:	MIDI Baudrate added for USART1 in USB CDC and MIDI mode
//REVNUM 4:	stability improved in sendUsbByte() and encodeMidi()
//			USART and USB buffer sizes optimized
//			USB EP2 fillstate level increased to 260 bytes
//			USART TX now interrupt driven
//			LED blink suppressed when MIDI Timing Clock or Active Sensing received
//			USERMESSAGES new: ifndef, no pushbuttons,potentiometers available
//new: option USB_CLOCK_SYNC (without quartz crystal)
//		(see top of modified file stm32f0xx.h !!!!!!!!!!!!!!)
//REVNUM 5:	 Option ALT_LED added.
//REVNUM 6	(25 MAY 2018): User Setup improved / some bugs fixed
//REVNUM 7	(03 JUNE 2018): structure of Flash Preset rearranged
//			quality and speed of (display setup list) improved
//			option "USERMESSAGES" removed, always active.
//REVNUM 8 (11 OCT 2021)
//			USB Enumeration improved
//			Flash storage improved
//			User interface ("CMD>") improved
//			internal handling of input change messages improved
//			new option VERO_PCB compatible with standard CFoth Veroboard PCB
//			PA13 now handles input change message on/off
//			input change msgs now PA1,4,5,6,7,PB1. VERO_PCB PA0 additionally
//			development migrated to Segger Embedded Studio for ARM 5.44
//released 24 Oct21
//REVNUM 9 (21 Nov 2021)
//			USB stability improved
//			user interface improved, decimal number base supported

// ##############################

#define REVNUM 9		//describes firmware revision

#define VERO_PCB		//USART TX level inverted insideF042, no ext.inverter
						//used for CForth compatible Veroboard version
						//PA0 additional I/O #7

#ifndef VERO_PCB
#define HARDWARE_VERSION 3  //describes project specific hardware version
//VERSION3: 74LVC2G14 as push/pull serial output driver
//          Circuit Board layout changed
#define IO_NUM	6		//number of inputs for change messages
#define ALT_LED           //LEDs OFF in idle state, ON when data transfer
#else
#define HARDWARE_VERSION 1  //describes project specific hardware version
//hardware as developed for CForth-F042, MIDI variant
#define IO_NUM	7		//number of inputs for change messages
#endif

//#define QUARTZ_OSC	//assembled with 8Mhz quartz crystal oscillator
						//only option, code for HSI48 removed
#define BUSPOWERED

#define DEFAULT_VIDH	0x04
#define DEFAULT_VIDL	0x83
#define DEFAULT_VID		((DEFAULT_VIDH << 8) | DEFAULT_VIDL)
#define DEFAULT_PIDH	0x57
#define DEFAULT_PIDL	0x40
#define DEFAULT_PID		((DEFAULT_PIDH << 8) | DEFAULT_PIDL)

#define CDC_CLASS   0
#define MIDI_CLASS  1

//#define SETUP_DBG  //sdbg

#ifdef SETUP_DBG
//  #define DETAIL
#endif

//USART communication
#ifdef SETUP_DBG
#define USART_RXBUFLEN 256
#else
 #define USART_RXBUFLEN 1024
#define USART_TXINTERRUPT
#endif
#ifdef USART_TXINTERRUPT
 #define USART_TXBUFLEN 768
#endif

#define EOX     0xF7
#define RUNNINGSTATE  //uncomment if not wanted
//works only on MIDI msgs re-transmitted serially
//for serially received MIDI msgs Running State is always supported

//bitmask shortcuts:
#define m0      1
#define m1      (1<<1)
#define m2      (1<<2)
#define m3      (1<<3)
#define m4      (1<<4)
#define m5      (1<<5)
#define m6      (1<<6)
#define m7      (1<<7)
#define m8      (1<<8)
#define m9      (1<<9)
#define m10     (1<<10)
#define m11     (1<<11)
#define m12     (1<<12)
#define m13     (1<<13)
#define m14     (1<<14)
#define m15     (1<<15)
#define m16     (1<<16)
#define m17     (1<<17)
#define m18     (1<<18)
#define m19     (1<<19)
#define m20     (1<<20)
#define m21     (1<<21)
#define m22     (1<<22)
#define m23     (1<<23)
#define m24     (1<<24)
#define m25     (1<<25)
#define m26     (1<<26)
#define m27     (1<<27)
#define m28     (1<<28)
#define m29     (1<<29)
#define m30     (1<<30)
#define m31     (1<<31)

#endif
