// USB-MidiCom  ***************************************************************
//
// Simple transparent data converter for STM32F042F6 (20 pin TSSOP package)
// based on USB MIDISTREAM and CDC-ABSTRACT CONTROL MODEL.
// Without any modifications, the code works with STM32F070F6
// (20 pin TSSOP package) EXCEPT the USB synchronized internal HSI48 oscillator.
//
// Additionally, a number of user defined MIDI messages can be released by
// attached pushbuttons or potentiometers.
//
// Author and (c)2016-21 Wolfgang Schemmert. The copyright claim and
// the GPL-3 Licence is restricted to this file and the depending files
// "main.h", "userinterface.c", "usb-fs-042.c" and "globaldefines.h".
// Other depending material is published and copyrighted
// by STMicroelectronics and Segger GmbH/Rowley Associates Ltd
//
// Contact: www.midi-and-more.de
// A detailled hardware construction and operation manual
// is provided at this website
//
// ****************************************************************************
//      This program is free software: you can redistribute it and/or modify
//      it under the terms of the GNU General Public License as published by
//      the Free Software Foundation, either version 3 of the License, or
//      (at your option) any later version.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//      Licence: GPL-3
//
// ****************************************************************************

#include "stm32f0xx.h"
#include "usb_regs.h"
#include "globaldefines.h"
#include "main.h"


int main(void)    // ********************************************************
// **************************************************************************
{
uint16_t recvword;
uint8_t recvbyte;
#ifdef USART_TXINTERRUPT
uint32_t fillstate;
#endif

    SCB->CCR |= SCB_CCR_STKALIGN_Msk; // Enable double word stack alignment
    //(recommended in Cortex-M3 r1p1, default in Cortex-M3 r2px and Cortex-M4)

    RCC->AHBENR |= RCC_AHBENR_GPIOAEN;      // Enable Port A clock
    RCC->AHBENR |= RCC_AHBENR_GPIOBEN;      // Enable Port B clock

// PB8 is used as red LED output
    GPIOB->MODER |= GPIO_MODER_MODER8_0 ; //PortB8 general purpose output mode
#ifdef ALT_LED
    GPIOB->BSRR = GPIO_BSRR_BR_8;         //Set bit PB8 (red LED OFF)
#else
    GPIOB->BSRR = GPIO_BSRR_BS_8;         //Set bit PB8 (red LED ON)
#endif
// PA0 is used as green LED output
    GPIOA->MODER |= GPIO_MODER_MODER0_0 ; //PortA0 general purpose output mode
#ifdef ALT_LED
   GPIOA->BSRR = GPIO_BSRR_BR_0;         //Set bit PA0 (green LED OFF)
#else
    GPIOA->BSRR = GPIO_BSRR_BS_0;         //Set bit PA0 (green LED ON)
#endif
    //GPIOx->OTYPER  is push-pull by default
    //GPIOx->OSPEEDR = 0;  //2 MHz by default
    //GPIOx->PUPDR  is HiZ by default

// for USB, remap PA11,12 to PA9,10
    RCC->APB2ENR |= RCC_APB2ENR_SYSCFGCOMPEN;
    SYSCFG->CFGR1 |= SYSCFG_CFGR1_PA11_PA12_RMP;

    SysTick_Config(SystemCoreClock/1000);

    gSYSCOUNT=20;
    gGREEN_LEDCOUNT=0;
    gRED_LEDCOUNT=0;
    gDELAYCOUNT=0;
    gTIMEOUT=0;
    gGUARDTIME=1000;
    gHGUARDTIME=1000;
    gStartSetup = 0 ;	//init detection of +++ sequence
    gUsbStartSetup = 0;
    gSetupMode = 0 ;
    gFlashChanged = 0;
	gNumBase = 1;		//default hex
	gExecOn = 0;
	gExecBlock = 0;
	gAdcOn = 0;
	gTest = 0;			//for debugging only
	gOver = 0;			//for debugging only

#ifndef SETUP_DBG
//start Independent Watchdog   *******************
//will be served at "pullUsartRxBuf" and "pullUsbRxBuf"
    IWDG->KR = 0x00005555;
    IWDG->RLR = 0x0000FFFF;
    msDelay(2);
    IWDG->PR = 0x00000002;      //resulting timeout ca. 1-2 seconds
    msDelay(2);
    IWDG->KR = 0x0000CCCC;      //start Independent Watchdog
#endif


//check jumper and set guDeviceClass  ******************
    GPIOA->PUPDR |=  GPIO_PUPDR_PUPDR14_0;  //does no harm to SWDebug
    GPIOA->PUPDR &=  ~GPIO_PUPDR_PUPDR14_1;
    msDelay(2);
//if PortA,14 = high (default): CDC_CLASS
//if PortA,14 = low (Jumper set): MIDI_CLASS
    guDeviceClass = CDC_CLASS;
    if ((GPIOA->IDR & m14) == 0)  guDeviceClass = MIDI_CLASS;

	guDeviceClass = CDC_CLASS;

//init I/O state  *********************
    popFlashPage31();           //copy user setup to SRAM for easier access

// init communication **************************
    initUSART();
	
#ifdef SETUP_DBG
    initSDBG();
#endif
	if (guDeviceClass == CDC_CLASS)  {
		msDelay(10);
		sendB('O');
		sendB('k');
		sendB(0x0D);
	}
    msDelay(10);

    initUSB();
	msDelay(500);
//check and store actual I/O state before enable automatic MIDI message
	
	configure_GPIO();
	gExecOn = 0;
	gExecOn = 1;
	for (recvbyte=0;recvbyte <= IO_NUM;recvbyte++) gInState[recvbyte] = 0;
	initADC();
	msDelay(100);
	MsgExecutor();		//is not sent because gExecOn is still = 0!

/*
//now check jumper and set I/O mode  ******************
    GPIOA->PUPDR |=  GPIO_PUPDR_PUPDR13_0;  //does no harm to SWDebug
    GPIOA->PUPDR &=  ~GPIO_PUPDR_PUPDR13_1;
    msDelay(2);
//if PortA,13 = high (default): no insertion of I/O messages
//if PortA,13 = low (Jumper set): activate I/O messages
    gExecOn = 0;
    if ((GPIOA->IDR & m10) == 0)  gExecOn = 0xFF;
*/
	gExecOn = 1;

    msDelay(10);

    while(1)
    {

       if ((guDeviceClass == CDC_CLASS) && gSetupMode) userSetup();
                                    //returns to mainloop when Exit

//start setup mode with 3 consecutive +++ with 1 sec guardtime before & after
//gStartSetup and gUsbStartSetup work as mutual exclude semaphore, too
        recvword = pullUsartRxBuf();
        if (recvword)  {
            recvbyte = (uint8_t)(recvword & 0x00FF);
#ifdef SETUP_DBG
		if (recvbyte != '~')  {
#endif
            if (guDeviceClass == CDC_CLASS) {
                if (gUsbStartSetup == 0)  {
                    if ((gStartSetup == 4) && (gGUARDTIME))  {
                        gStartSetup = 0;
                    }
                    if ((recvbyte == '+') && (gStartSetup == 2)) {
                        gStartSetup = 3;
                        gGUARDTIME = 1000;      //milliseconds
                        sendB('+');
                    }
                    if ((recvbyte == '+') && (gStartSetup == 1)) {
                        gStartSetup = 2;
                        gGUARDTIME = 400;
                        sendB('+');
                    }
                    if ((recvbyte == '+')&&(gStartSetup==0)&&(gGUARDTIME==0)) {
                        gStartSetup = 1;
                        gGUARDTIME = 400;
                        sendB('+');
                    }
                    if (gSetupMode == 0) {
                        sendUsbByte(recvbyte);
                        if (gStartSetup == 0) gGUARDTIME = 1000;
                    }
                }
                else sendUsbByte(recvbyte);
            }
            else enCodeMidi((uint8_t)(recvbyte&0x00FF));

#ifdef SETUP_DBG
		}
        else  sendDebug();
//Tilde char ~ transmits and flushes the debug buffer.
//It transmits and flushes automatically when it is full.
#endif

        }
//added 01 Dec 2017 / REVNUM 4,
//else interrupt driven USART Tx would overflow at high USB data load
//calculate fill level of gUSART_TXBUF
//TAIL>HEAD? then add USART_TXBUFLEN to get the absolute difference
#ifdef USART_TXINTERRUPT
        if (gTXTAIL > gTXHEAD) {
            fillstate = (uint32_t)gTXHEAD + USART_TXBUFLEN - (uint32_t)gTXTAIL;
        }
        else fillstate = (uint32_t)gTXHEAD - (uint32_t)gTXTAIL ;

        if (guConfigured && (fillstate < (USART_TXBUFLEN-80))) {
#endif
            recvword = pullUsbRxBuf();
            if (recvword)  {
                recvbyte = (uint8_t)(recvword & 0x00FF);
                if (guDeviceClass == CDC_CLASS) {
                    if (gStartSetup == 0)  {
                        if ((gUsbStartSetup == 4) && (gHGUARDTIME)) {
                            gUsbStartSetup = 0;
                        }
                        if((recvbyte == '+') && (gUsbStartSetup == 2))  {
                            gUsbStartSetup = 3;
                            gHGUARDTIME = 1000;      //milliseconds
                            sendUsbByte('+');
                        }
                        if ((recvbyte == '+') && (gUsbStartSetup == 1))  {
                            gUsbStartSetup = 2;
                            gHGUARDTIME = 400;
                            sendUsbByte('+');
                        }
                      if ((recvbyte=='+')&&(gUsbStartSetup==0)&&(gHGUARDTIME==0)){
                            gUsbStartSetup = 1;
                            gHGUARDTIME = 400;
                            sendUsbByte('+');
                        }
                        if (gSetupMode == 0) {
                            sendB(recvbyte);
                            if (gUsbStartSetup == 0) gHGUARDTIME = 1000;
                        }
                    }
                }
                else  sendB(recvbyte);
            }
#ifdef USART_TXINTERRUPT
        }
#endif
    }
}


void configure_GPIO(void)  // ***********************************************
// **************************************************************************
{
uint32_t k;
/*
uint16_t *pointer;
	pointer = (uint16_t *)&(gFlashMirror.ComBaud);
	sendCRLF();
	sendAllB('%');
	sendAllB('%');
	sendAllB('%');
	sendAllB('%');
	sendCRLF();
	for (k= 0; k<8;k++) {
		sendAllX16(*(pointer + k));
		sendCRLF();
	}

	sendAllB('#');
	sendAllB('#');
	sendAllB('#');
	sendAllB('#');
	sendCRLF();
*/
	for (k= 1; k <= IO_NUM;k++) {
//		pointer = &(gRuntimeSetup.PidH) +(k<<1);
//		sendAllX16(*pointer++);
//		sendAllX16(*pointer);
//		sendCRLF();
		configurePort(k);
	}
}


void configurePort(uint32_t number)  // *************************************
// built as subroutine to supply service to "usb-fs-042.c" hardware independent
// **************************************************************************
{
#ifdef VERO_PCB
	switch(number)  {
	case 1:
//init I/O state
//init pull-up resistor for any user accessible digital I/O
//i.e. default digital input stage = '1'
		GPIOA->MODER &= ~GPIO_MODER_MODER4 ;
		GPIOA->PUPDR &= ~GPIO_PUPDR_PUPDR4_1;
		GPIOA->PUPDR |=  GPIO_PUPDR_PUPDR4_0;
//when configured as "analog input" initalize differently:
		if ((gFlashMirror.Msg1D & 0x8000) == 0x8000) { 
			GPIOA->MODER |= GPIO_MODER_MODER4;			//analog
			GPIOA->PUPDR &= ~(GPIO_PUPDR_PUPDR4);		//HiZ
		}
		break;
	case 2:
		GPIOA->MODER &= ~GPIO_MODER_MODER1 ;
		GPIOA->PUPDR &= ~GPIO_PUPDR_PUPDR1_1;
		GPIOA->PUPDR |=  GPIO_PUPDR_PUPDR1_0;
		if ((gFlashMirror.Msg2D & 0x8000) == 0x8000) {
			GPIOA->MODER |= GPIO_MODER_MODER1;
			GPIOA->PUPDR &= ~GPIO_PUPDR_PUPDR1;
		}
		break;
	case 3:
		GPIOA->MODER &= ~GPIO_MODER_MODER0 ;
		GPIOA->PUPDR &= ~GPIO_PUPDR_PUPDR0_1;
		GPIOA->PUPDR |=  GPIO_PUPDR_PUPDR0_0;
		if ((gFlashMirror.Msg3D & 0x8000) == 0x8000) {
			GPIOA->MODER |= GPIO_MODER_MODER0;
			GPIOA->PUPDR &= ~GPIO_PUPDR_PUPDR0;
		}
		break;
	case 4:
		GPIOA->MODER &= ~GPIO_MODER_MODER5 ;
		GPIOA->PUPDR &= ~GPIO_PUPDR_PUPDR5_1;
		GPIOA->PUPDR |=  GPIO_PUPDR_PUPDR5_0;
		if ((gFlashMirror.Msg4D & 0x8000) == 0x8000) {
			GPIOA->MODER |= GPIO_MODER_MODER5;
			GPIOA->PUPDR &= ~GPIO_PUPDR_PUPDR5;
		}
		break;
	case 5:
		GPIOA->MODER &= ~GPIO_MODER_MODER6 ;
		GPIOA->PUPDR &= ~GPIO_PUPDR_PUPDR6_1;
		GPIOA->PUPDR |=  GPIO_PUPDR_PUPDR6_0;
		if ((gFlashMirror.Msg5D & 0x8000) == 0x8000) {
			GPIOA->MODER |= GPIO_MODER_MODER6;
			GPIOA->PUPDR &= ~GPIO_PUPDR_PUPDR6;
		}
		break;
	case 6:
		GPIOA->MODER &= ~GPIO_MODER_MODER7 ;
		GPIOA->PUPDR &= ~GPIO_PUPDR_PUPDR7_1;
		GPIOA->PUPDR |=  GPIO_PUPDR_PUPDR7_0;
		if ((gFlashMirror.Msg6D & 0x8000) == 0x8000) {
			GPIOA->MODER |= GPIO_MODER_MODER7;
			GPIOA->PUPDR &= ~GPIO_PUPDR_PUPDR7;
		}
	case 7:
		GPIOB->MODER &= ~GPIO_MODER_MODER1 ;
		GPIOB->PUPDR &= ~GPIO_PUPDR_PUPDR1;
		GPIOB->PUPDR |=  GPIO_PUPDR_PUPDR1_0;
		if ((gFlashMirror.Msg7D & 0x8000) == 0x8000) {
			GPIOB->MODER |= GPIO_MODER_MODER1;
			GPIOB->PUPDR &= ~GPIO_PUPDR_PUPDR1;
		}
		break;
	}

#else

	switch(number)  {
	case 1:
//init I/O state
//init pull-up resistor for any user accessible digital I/O
//i.e. default digital input stage = '1'
		GPIOA->MODER &= ~GPIO_MODER_MODER1 ;
		GPIOA->PUPDR &= ~GPIO_PUPDR_PUPDR1_1;
		GPIOA->PUPDR |=  GPIO_PUPDR_PUPDR1_0;
//when configured as "analog input" initalize differently:
		if ((gFlashMirror.Msg1D & 0x8000) == 0x8000) { 
			GPIOA->MODER |= GPIO_MODER_MODER1;			//analog
			GPIOA->PUPDR &= ~(GPIO_PUPDR_PUPDR1);		//HiZ
		}
		break;
	case 2:
		GPIOA->MODER &= ~GPIO_MODER_MODER4 ;
		GPIOA->PUPDR &= ~GPIO_PUPDR_PUPDR4_1;
		GPIOA->PUPDR |=  GPIO_PUPDR_PUPDR4_0;
		if ((gFlashMirror.Msg2D & 0x8000) == 0x8000) {
			GPIOA->MODER |= GPIO_MODER_MODER4;
			GPIOA->PUPDR &= ~GPIO_PUPDR_PUPDR4;
		}
		break;
	case 3:
		GPIOA->MODER &= ~GPIO_MODER_MODER5 ;
		GPIOA->PUPDR &= ~GPIO_PUPDR_PUPDR5_1;
		GPIOA->PUPDR |=  GPIO_PUPDR_PUPDR5_0;
		if ((gFlashMirror.Msg3D & 0x8000) == 0x8000) {
			GPIOA->MODER |= GPIO_MODER_MODER5;
			GPIOA->PUPDR &= ~GPIO_PUPDR_PUPDR5;
		}
		break;
	case 4:
		GPIOA->MODER &= ~GPIO_MODER_MODER6 ;
		GPIOA->PUPDR &= ~GPIO_PUPDR_PUPDR6_1;
		GPIOA->PUPDR |=  GPIO_PUPDR_PUPDR6_0;
		if ((gFlashMirror.Msg4D & 0x8000) == 0x8000) {
			GPIOA->MODER |= GPIO_MODER_MODER6;
			GPIOA->PUPDR &= ~GPIO_PUPDR_PUPDR6;
		}
		break;
	case 5:
		GPIOA->MODER &= ~GPIO_MODER_MODER7 ;
		GPIOA->PUPDR &= ~GPIO_PUPDR_PUPDR7_1;
		GPIOA->PUPDR |=  GPIO_PUPDR_PUPDR7_0;
		if ((gFlashMirror.Msg5D & 0x8000) == 0x8000) {
			GPIOA->MODER |= GPIO_MODER_MODER7;
			GPIOA->PUPDR &= ~GPIO_PUPDR_PUPDR7;
		}
		break;
	case 6:
		GPIOB->MODER &= ~GPIO_MODER_MODER1 ;
		GPIOB->PUPDR &= ~GPIO_PUPDR_PUPDR1;
		GPIOB->PUPDR |=  GPIO_PUPDR_PUPDR1_0;
		if ((gFlashMirror.Msg6D & 0x8000) == 0x8000) {
			GPIOB->MODER |= GPIO_MODER_MODER1;
			GPIOB->PUPDR &= ~GPIO_PUPDR_PUPDR1;
		}
	}
#endif
}


//##########################################################################
// Misc Low Level Routines
//************************
void LED_Off(void)  // ******************************************************
// built as subroutine to supply service to "usb-fs-042.c" hardware independent
// **************************************************************************
{
        GPIOB->BSRR = GPIO_BSRR_BR_8; // clear bit PB8,LED OFF
#ifdef VERO_PCB
        gGREEN_LEDCOUNT = 250; //milliseconds
#else
        gRED_LEDCOUNT = 250; //milliseconds
#endif
}


void msDelay(uint16_t count)   // *******************************************
//blocking delay by 'count' ms
//countdown by SysTick handler
// **************************************************************************
{
    gDELAYCOUNT = count;
    while(gDELAYCOUNT)  IWDG->KR = 0x0000AAAA; //feed watchdog
}


void servSystem(void)   // **************************************************
//not very time critical but permanently repeated functions for system control
//exclusively called from pullUsartRxBuf and pullUsbRxBuf
//this way it is regularily served when idle waiting for external input
//but watchdog resets when it hangs somewhere else
//additionally triggered by SysTick 50 times/second
// ****************************************************************************
{
static uint32_t secCount;

    gSYSCOUNT = 20;         //restart 50Hz service interval
//feed watchdog
    IWDG->KR = 0x0000AAAA;  //timeout ca. 1-2 seconds

//interpret I/O status and send MIDI messages
	MsgExecutor();	//state evaluation is performed even if gExec = 0

//LED control
#ifdef VERO_PCB
    if (gGREEN_LEDCOUNT==0) GPIOB->BSRR = GPIO_BSRR_BS_8; //Set bit PB8(LED ON)
#else
  #ifdef ALT_LED
    if (gRED_LEDCOUNT==0)  GPIOB->BSRR = GPIO_BSRR_BR_8;  //Set bit PB8(LED OFF)
    if (gGREEN_LEDCOUNT==0) GPIOA->BSRR = GPIO_BSRR_BR_0; //Set bit PA0(LED OFF)
  #else
    if (gRED_LEDCOUNT==0)  GPIOB->BSRR = GPIO_BSRR_BS_8; // Set bit PB8 (LED ON)
    if (gGREEN_LEDCOUNT==0) GPIOA->BSRR = GPIO_BSRR_BS_0;// Set bit PA0 (LED ON)
  #endif
#endif
	
	secCount++;
	if (secCount >= 50)  {
		secCount = 0;
		if (gRuntimeSetup.Channel == 0xFFFF) gGlobalMidiChannel = 0;
		else gGlobalMidiChannel = (uint8_t)gRuntimeSetup.Channel;

#ifndef SETUP_DBG
		if (((GPIOA->IDR & m14) == 0) && (guDeviceClass == CDC_CLASS))  {
			stopUSB();
			guDeviceClass = MIDI_CLASS;
			initUSART();
			msDelay(500);
			initUSB();

		}
		else if ((GPIOA->IDR & m14) && (guDeviceClass == MIDI_CLASS))  {
			stopUSB();
			guDeviceClass = CDC_CLASS;
			initUSART();
			msDelay(500);
			initUSB();
		}
#endif
		if (gExecBlock == 0)  {
			if (((GPIOA->IDR & m13) == 0) && (gExecOn == 0))  {
				gExecOn = 0xFF;     //input change messages are activated
			}
			if ((GPIOA->IDR & m13) && gExecOn)  {
				gExecOn = 0;		//input change messages deactivated
			}
		}
	}
	if (gExecBlock) gExecOn = 0;
	else gExecOn = 1;
}


void MsgExecutor(void)   //***********************************************
//read analog inputs PA1,PA4,PA5,PA6 and compare with previous value
//and/or read digital levels of ports A and B and compare with previous value
//send MIDI messages as configured in gRuntimeSetup
// gRuntimeSetup contains operative data, possibly temporarily updated)
// **************************************************************************
{
uint16_t analogLevel, digitalState;

	if ((ADC1->CR & ADC_CR_ADEN) == 0) initADC();	//repeted for stability

#ifdef VERO_PCB
	if (gFlashMirror.Msg1D != 0xFFFF) {
		if (((gFlashMirror.Msg1D & 0x8000) == 0x8000) && gAdcOn) {
			analogLevel = readADC(4);				//PA4 is ADC_IN4
//			sendAllX16(analogLevel);
			debounceAnalog(1,analogLevel);
		}
		else  {
			if (GPIOA->IDR & m4) digitalState = 1;		
			else digitalState = 0;
			debounceDigital(1,digitalState);
		}
	}

	if (gFlashMirror.Msg2D != 0xFFFF) {
		if (((gFlashMirror.Msg2D & 0x8000) == 0x8000) && gAdcOn) {
			analogLevel = readADC(1);				//PA1 is ADC_IN1
			debounceAnalog(2,analogLevel);
		}
		else  {
			if (GPIOA->IDR & m1) digitalState = 1;		
			else digitalState = 0;		
			debounceDigital(2,digitalState);
		}
	}

	if (gFlashMirror.Msg3D != 0xFFFF) {
		if (((gFlashMirror.Msg3D & 0x8000) == 0x8000) && gAdcOn) {
			analogLevel = readADC(0);				//PA0 is ADC_IN0
			debounceAnalog(3,analogLevel);
		}
		else  {
			if (GPIOA->IDR & m0) digitalState = 1;		
			else digitalState = 0;		
			debounceDigital(3,digitalState);
		}
	}

	if (gFlashMirror.Msg4D != 0xFFFF) {
		if (((gFlashMirror.Msg4D & 0x8000) == 0x8000) && gAdcOn) {
			analogLevel = readADC(5);				//PA5 is ADC_IN5
			debounceAnalog(4,analogLevel);
		}
		else  {
			if (GPIOA->IDR & m5) digitalState = 1;		
			else digitalState = 0;		
			debounceDigital(4,digitalState);
		}
	}

	if (gFlashMirror.Msg5D != 0xFFFF) {
		if (((gFlashMirror.Msg5D & 0x8000) == 0x8000) && gAdcOn) {
			analogLevel = readADC(6);				//PA6 is ADC_IN6
			debounceAnalog(5,analogLevel);
		}
		else  {
			if (GPIOA->IDR & m6) digitalState = 1;		
			else digitalState = 0;		
			debounceDigital(5,digitalState);
		}
	}

	if (gFlashMirror.Msg6D != 0xFFFF) {
		if (((gFlashMirror.Msg7D & 0x8000) == 0x8000) && gAdcOn) {
			analogLevel = readADC(7);				//PA7 is ADC_IN7
			debounceAnalog(6,analogLevel);
		}
		else  {
			if (GPIOA->IDR & m7) digitalState = 1;		
			else digitalState = 0;
			debounceDigital(6,digitalState);
		}
	}
	if (gFlashMirror.Msg7D != 0xFFFF) {
		if (((gFlashMirror.Msg6D & 0x8000) == 0x8000) && gAdcOn) {
			analogLevel = readADC(9);				//PB1 is ADC_IN9
			debounceAnalog(7,analogLevel);
		}
		else  {
			if (GPIOB->IDR & m1) digitalState = 1;		
			else digitalState = 0;
			debounceDigital(7,digitalState);
		}
	}

#else
	if (gFlashMirror.Msg1D != 0xFFFF) {
		if (((gFlashMirror.Msg1D & 0x8000) == 0x8000) && gAdcOn) {
			analogLevel = readADC(1);				//PA1 is ADC_IN1
			debounceAnalog(1,analogLevel);
		}
		else  {
			if (GPIOA->IDR & m1) digitalState = 1;		
			else digitalState = 0;		
			debounceDigital(1,digitalState);
		}
	}

	if (gFlashMirror.Msg2D != 0xFFFF) {
		if (((gFlashMirror.Msg2D & 0x8000) == 0x8000) && gAdcOn) {
			analogLevel = readADC(4);				//PA4 is ADC_IN4
			debounceAnalog(2,analogLevel);
		}
		else  {
			if (GPIOA->IDR & m4) digitalState = 1;		
			else digitalState = 0;		
			debounceDigital(2,digitalState);
		}
	}

	if (gFlashMirror.Msg3D != 0xFFFF) {
		if (((gFlashMirror.Msg3D & 0x8000) == 0x8000) && gAdcOn) {
			analogLevel = readADC(5);				//PA5 is ADC_IN5
			debounceAnalog(3,analogLevel);
		}
		else  {
			if (GPIOA->IDR & m5) digitalState = 1;		
			else digitalState = 0;		
			debounceDigital(3,digitalState);
		}
	}

	if (gFlashMirror.Msg4D != 0xFFFF) {
		if (((gFlashMirror.Msg4D & 0x8000) == 0x8000) && gAdcOn) {
			analogLevel = readADC(6);				//PA6 is ADC_IN6
			debounceAnalog(4,analogLevel);
		}
		else  {
			if (GPIOA->IDR & m6) digitalState = 1;		
			else digitalState = 0;		
			debounceDigital(4,digitalState);
		}
	}

	if (gFlashMirror.Msg5D != 0xFFFF) {
		if (((gFlashMirror.Msg5D & 0x8000) == 0x8000) && gAdcOn) {
			analogLevel = readADC(7);				//PA7 is ADC_IN7
			debounceAnalog(5,analogLevel);
		}
		else  {
			if (GPIOA->IDR & m7) digitalState = 1;		
			else digitalState = 0;		
			debounceDigital(5,digitalState);
		}
	}

	if (gFlashMirror.Msg6D != 0xFFFF) {
		if (((gFlashMirror.Msg6D & 0x8000) == 0x8000) && gAdcOn) {
			analogLevel = readADC(9);				//PB1 is ADC_IN9
			debounceAnalog(6,analogLevel);
		}
		else  {
			if (GPIOB->IDR & m1) digitalState = 1;		
			else digitalState = 0;
			debounceDigital(6,digitalState);
		}
	}
#endif
}


void debounceAnalog(uint8_t number, uint16_t level)  // *********************
// **************************************************************************
{
uint16_t * structPtr;
uint16_t hysteresis, statusType;

//if not "high res Pitch Wheel Change" - reduce analogLevel to 7 bit format
//in struct gstructPtr the offset relative to PidH is =(user I/Onumber *2)
    structPtr = &(gRuntimeSetup.PidH) + (number<<1);  // *2
	if (*structPtr == 0xFFFF) return ;				//message not defined
	if (*(structPtr+1) == 0xFFFF) return ;			//message not defined

	hysteresis = gRuntimeSetup.Hysteresis;
	statusType = *structPtr;
	if ((statusType & 0x00FF) > 0x000F)  statusType = statusType >> 4;

    if ((statusType != 0x000E)  ||
			((*(structPtr+1) & 0xFF00 )!= 0x8100)) {//different debounce level
		level = level>>5;					//reduce to 7 bit resolution
		if (hysteresis) hysteresis -= 1;	//reduce for low resolution msg
    }

    if (gInTest[number] != level) {
        if (gHysteresis[number])  {     //last change was towards lower level
            if (level < gInTest[number]) {
				gInState[number] = level;
				if (gExecOn) sendAnaMsg(number);
                gInTest[number] = level;
            }
            if (level > gInTest[number] + hysteresis) {
                                        //direction change,apply hysteresis
				gInState[number] = level;
                if (gExecOn) sendAnaMsg(number);
                gInTest[number] = level;
                gHysteresis[number] = 0;
            }
        }
        else {                          //last change was towards higher level
            if (level > gInTest[number]) {
				gInState[number] = level;
                if (gExecOn) sendAnaMsg(number);
                gInTest[number] = level;
            }
            if (level < gInTest[number] - hysteresis) {
                                        //direction change,apply hysteresis
				gInState[number] = level;
                if (gExecOn) sendAnaMsg(number);
                gInTest[number] = level;
                gHysteresis[number] = 0xFF;
            }
        }
    }
}


void debounceDigital(uint8_t number,uint16_t state)  // *********************
// **************************************************************************
{
uint16_t *structPtr;
uint8_t level;

    if ((state) != gInTest[number])  gInTest[number] = state;
    else {                                  //debounce for one service period
        if (state != gInState[number])  {
            gInState[number] = state;
			if (gExecOn)  {
				sendDigMsg(number);
            }
        }
    }
}


uint16_t readADC(uint8_t inport)  // ****************************************
//because Vdda is filtered very badly, improve result by oversampling/average
//one subroutine call takes about 340us
// **************************************************************************
{
uint32_t  result, correction;
uint8_t k;

//    GPIOB->BSRR = GPIO_BSRR_BR_8; //  LED OFF

    ADC1->CHSELR = (uint32_t)(1<<inport);
    ADC1->CR |= ADC_CR_ADSTART;
    while ((ADC1->ISR & ADC_ISR_EOC) == 0) ;
    result = (uint32_t)ADC1->DR;

    for (k= 0; k<65; k++)  {   //for an exact average, the limit should be 63
    //small stretch-up because some potentiometers cannot scan the full range
        ADC1->CHSELR = (uint32_t)(1<<inport);
        ADC1->CR |= ADC_CR_ADSTART;
        while ((ADC1->ISR & ADC_ISR_EOC) == 0) ;
        result += (uint32_t)ADC1->DR;
    }
    correction = result & 0x0000000F;
    result = result >>4;                    //reduce resolution to 12 bit
    if (correction > 0x00000007) result++;  //commercial roundup
    result -= 64 ; //3;    //compensate potentiometer with residual end resistor
    if  (result > 0xFF000000) result = 0;   //limit compensation underflow
    if (result > 0x00000FFF) result = 0x00000FFF; //limit overflow

//    GPIOB->BSRR = GPIO_BSRR_BS_8; // LED ON

    return (uint16_t)result;
}


void initADC(void)   // *****************************************************
// **************************************************************************
{
    RCC->APB2ENR |= RCC_APB2ENR_ADC1EN;
    ADC1->CFGR2 = ADC_CFGR2_CKMODE_1; //CKMODE=10,clock source=APB2_CLK/4 =12MHz
    ADC1->CR |= ADC_CR_ADEN;
    while ((ADC1->ISR & ADC_ISR_ADRDY) == 0) ;

    ADC1->CFGR1 = 1<<3;         //right aligned, 10bit resolution
    ADC->CCR = 0;               //no special ADC channels active
    ADC1->SMPR = 3;             //conversion time about 3.5us
	gAdcOn = 0xFF;
}


void sendAnaMsg(uint8_t number)   // ****************************************
// **************************************************************************
{
uint8_t status, data1, data2;
uint16_t value, msgS, msgD;
uint16_t *structPtr;

//in struct gFlashMirror the offset relative to PidH is =(user I/Onumber*2)
    structPtr = &(gRuntimeSetup.PidH) + (number<<1);  // *2
 	if (*structPtr == 0xFFFF) return;		//no message configured 
 	msgS = *structPtr & 0x00FF;
	if (msgS > 0xF3) return;		//1 byte MIDI messages,i.e.bad user setup
    status = (uint8_t)msgS;
    if (status <= 0x0F) status = (status << 4) | gGlobalMidiChannel;
	msgS = status >> 4;						//reduce to status type
	structPtr++;
	msgD = *structPtr;
	if (msgD == 0xFFFF) return;				//no message configured 

    if (guDeviceClass == MIDI_CLASS) sendEncodeB(status);
	
	value = gInState[number];
     if (msgS == 0x000E)  {					//pitch change
		if ((msgD & 0xFF00) == 0x8100)  {	//12 bit analog state message
			data1 = (uint8_t) ((value & 0x001F)<<2);
								//least significant 5 bits->7bit left aligned
			data2 = (uint8_t) (value >> 5);	//=most significant 7bits
			value |= 0x8000;				//discriminator for sendAsciiMsg
		}
		else  {								//7 bit analog state message
			data1 = 0;						//Pitch Wheel LSB=0								
			data2 = (uint8_t)(value & 0x007F);
										//reduced to 7bit in calling routine
		}
		if (guDeviceClass == MIDI_CLASS)  {
			sendEncodeB(data1) ;
			sendEncodeB(data2 & 0x7F) ;
		}
		else sendAsciiMsg(status, number, value);
	}
    else if ((msgS < 0x000C) || (status == 0xF2)) {
										//send NotePitch or Controller Num
		data1 = (uint8_t)(*structPtr & 0x007F);
		data2 = (uint8_t)(value & 0x007F);
										//reduced to 7bit in calling routine
		if (guDeviceClass == MIDI_CLASS)  {
			sendEncodeB(data1) ;
			sendEncodeB(data2) ;
		}
		else sendAsciiMsg(status, number,(uint16_t)data2);
    }
    else if ((msgS == 0x000C) || (msgS == 0x000D) || (status == 0xF3)) {
										//send NotePitch or Controller Num
		data2 = (uint8_t)(value & 0x007F);
		if (guDeviceClass == MIDI_CLASS)  {
			sendEncodeB(data2) ;
		}
		else sendAsciiMsg(status, number,(uint16_t)data2);
    }
	else  {				//measure of precaution,should be filtered off before
		if (guDeviceClass == MIDI_CLASS)  {
			sendEncodeB(0);
			sendEncodeB(0);
		}
		else sendAsciiMsg('E', status, 0);
	}
}


void sendDigMsg(uint8_t number)   // ****************************************
// **************************************************************************
{
uint8_t status, data1, data2;
uint16_t  msgS;
uint16_t *structPtr;

//in struct gFlashMirror the offset relative to PidH is=(user I/Onumber *2)
    structPtr = &(gRuntimeSetup.PidH) + (number<<1);  // *2
    msgS = (*structPtr++ & 0x00FF);

    if ((msgS >= 0x00F6) && (msgS != 0x00F9) && (msgS != 0x00FD)) {
		if (gInState[number] == 0) {
			//single byte message - send only when pushbuttton is pressed
			if (guDeviceClass == MIDI_CLASS) sendEncodeB(msgS);
			else  sendAsciiMsg(0, number, msgS);
		}
		return;
	}

    status = (uint8_t)msgS;
    if (status <= 0x0F) status = (status << 4) | gGlobalMidiChannel;
    if (guDeviceClass == MIDI_CLASS) sendEncodeB(status);
    msgS = status >> 4;						//reduce to status type

	if (msgS == 0x000E) data1 = 0;
	else data1 = (uint8_t)(*structPtr & 0x007F);
	if (gInState[number])	data2 = 0;			//button NOT pushed: send 0
	else data2 =(uint8_t)((*structPtr >> 8) & 0x007F); //pushed=config value

	if ((msgS == 0x000C) || (msgS == 0x000D) || (status == 0xF3))  {
		if (guDeviceClass == MIDI_CLASS)  {
			sendEncodeB(data2);
		}
		else sendAsciiMsg(0,number,data2);	//digital msg with user I/O number
    }
	else if (((msgS >= 0x0009) && (msgS <= 0x000B)) || (status == 0xF2))  {
		if (guDeviceClass == MIDI_CLASS)  {
			sendEncodeB(data1);
			sendEncodeB(data2);
		}
		else sendAsciiMsg(0,number,data2);	//digital msg with user I/O number
    }
	else  {				//measure of precaution,should be filtered off before
		if (guDeviceClass == MIDI_CLASS)  {
			sendEncodeB(0);
			sendEncodeB(0);
		}
		else sendAsciiMsg('E', status, 0);
	}
}


void sendAsciiMsg(uint8_t mode, uint8_t number, uint16_t value) // **********
// **************************************************************************
{
const uint8_t errorStr[] = "Error:" ;

	sendAllB(' ');
	if (mode == 0) {
		sendAllB('#');
		if (number == 17) sendAllB('u');
		else if (number >= 10)  {
			sendAllB('1');
			sendAllB(number + 0x26);	//=number-10
		}
		else sendAllB(number + 0x30);
		sendAllB('d');

		if (value == 0) sendAllB('0');
		else  {
			if (gNumBase)  {
				sendAllB('0');
				sendAllB('x');
				sendAllX8((uint8_t)(value & 0x007F));
			}
			else  sendAllS12(value & 0x007F);
		}
	}
	else if (mode == 'E') {
		sendAllString(errorStr);
		sendAllB('#');
		if (number == 17) sendAllB('u');
		else if (number >= 10)  {
			sendAllB('1');
			sendAllB(number + 0x26);	//=number-10
		}
		else sendAllB(number + 0x30);
		sendAllB('E');
		sendAllB('r');
		sendAllB('r');
	}
	else {
		sendAllB('#');
		if (number == 17) sendAllB('u');
		else if (number >= 10)  {
			sendAllB('1');
			sendAllB(number + 0x26);	//=number-10
		}
		else sendAllB(number + 0x30);
		sendAllB('a');
		if ((mode & 0xF0) == 0xE0) {
			if (value & 0x8000)  {
				sendAllB('h');
			if (gNumBase)  {
				sendAllB('0');
				sendAllB('x');
				sendAllX12(value & 0x0FFF);
			}
			else  sendAllS12(value & 0x0FFF);
			}
			else {
			if (gNumBase)  {
				sendAllB('0');
				sendAllB('x');
				sendAllX8((uint8_t)(value & 0x007F));
			}
			else  sendAllS12(value & 0x007F);
			}
		}
		else  {
			if (gNumBase)  {
				sendAllB('0');
				sendAllB('x');
				sendAllX8((uint8_t)(value & 0x007F));
			}
			else  sendAllS12(value & 0x007F);
		}
	}
	sendAllB(' ');
}

// ##########################################################################
//Flash operations  

void popFlashPage31(void)   //***********************************************
// **************************************************************************
{
uint8_t k;
uint32_t *flashPtr;
uint16_t * fMirrorPtr, * runtimeSetupPtr;

    flashPtr = (uint32_t *)Page31;
    fMirrorPtr = (uint16_t *) &(gFlashMirror.ComBaud);
    runtimeSetupPtr = (uint16_t *) &(gRuntimeSetup.ComBaud);

    for (k=0;k<12;k++)  {
        *fMirrorPtr = (uint16_t)(*flashPtr & 0x0000FFFF);
        *runtimeSetupPtr = *fMirrorPtr;
        fMirrorPtr++;
		runtimeSetupPtr++;
        *fMirrorPtr = (uint16_t)((*flashPtr >> 16) & 0x0000FFFF);
        *runtimeSetupPtr = *fMirrorPtr;
        fMirrorPtr++;
		runtimeSetupPtr++;
        flashPtr++;
    }
	if (gFlashMirror.ComBaud == 0xFFFF) {
		gFlashMirror.ComBaud = 11;
		gRuntimeSetup.ComBaud = 11;
	}
	if (gFlashMirror.MidiBaud == 0xFFFF) {
		gFlashMirror.MidiBaud = 31; //31
		gRuntimeSetup.MidiBaud = 31; //31
	}
	if (gFlashMirror.Channel == 0xFFFF) {
		gFlashMirror.Channel = 0;
		gRuntimeSetup.Channel = 0;
	}
	if (gFlashMirror.Hysteresis == 0xFFFF) {
		gFlashMirror.Hysteresis = 0x0002;			//default best for 
		gRuntimeSetup.Hysteresis = 0x0002;			//most applications
	}
	if ((gFlashMirror.VidH == 0xFFFF) || (gFlashMirror.VidL == 0xFFFF)) {
		gFlashMirror.VidH = DEFAULT_VIDH;
		gFlashMirror.VidL = DEFAULT_VIDL;
		gRuntimeSetup.VidH = DEFAULT_VIDH;
		gRuntimeSetup.VidL = DEFAULT_VIDL;
	}
	if ((gFlashMirror.PidH == 0xFFFF) || (gFlashMirror.PidL == 0xFFFF)) {
		gFlashMirror.PidH = DEFAULT_PIDH;
		gFlashMirror.PidL = DEFAULT_PIDL;
		gRuntimeSetup.PidH = DEFAULT_PIDH;
		gRuntimeSetup.PidL = DEFAULT_PIDL;
	}
	if ((gFlashMirror.Msg1S & gFlashMirror.Msg1D) == 0xFFFF) {
		gFlashMirror.Msg1S = 0x000B; //0x000B
		gRuntimeSetup.Msg1S = gFlashMirror.Msg1S;	//default CtrlCh
		gFlashMirror.Msg1D = 0x4001;				//data1 =input number
		gRuntimeSetup.Msg1D = 0x4001;				//bit15=A/D msg
	}
	if ((gFlashMirror.Msg2S & gFlashMirror.Msg2D) == 0xFFFF) {
		gFlashMirror.Msg2S = 0x000B;
		gRuntimeSetup.Msg2S = gFlashMirror.Msg2S;	//default CtrlCh
		gFlashMirror.Msg2D = 0x4002;				//data1 =input number
		gRuntimeSetup.Msg2D = 0x4002;
	}
	if ((gFlashMirror.Msg3S & gFlashMirror.Msg3D) == 0xFFFF) {
		gFlashMirror.Msg3S = 0x000B;
		gRuntimeSetup.Msg3S = gFlashMirror.Msg3S;	//default CtrlCh
		gFlashMirror.Msg3D = 0x4003;				//data1 =input number
		gRuntimeSetup.Msg3D = 0x4003;				//data2 =0x40
	}
	if ((gFlashMirror.Msg4S & gFlashMirror.Msg4D) == 0xFFFF) {
		gFlashMirror.Msg4S = 0x000B;
		gRuntimeSetup.Msg4S = gFlashMirror.Msg4S;	//default CtrlCh
		gFlashMirror.Msg4D = 0x4004;				//data1 =input number
		gRuntimeSetup.Msg4D = 0x4004;				//data2 =0x40
	}
	if ((gFlashMirror.Msg5S & gFlashMirror.Msg5D) == 0xFFFF) {
		gFlashMirror.Msg5S = 0x000B;
		gRuntimeSetup.Msg5S = gFlashMirror.Msg5S;	//default CtrlCh
		gFlashMirror.Msg5D = 0x4005;				//data1 =input number
		gRuntimeSetup.Msg5D = 0x4005;				//data2 =0x40
	}
	if ((gFlashMirror.Msg6S & gFlashMirror.Msg6D) == 0xFFFF) {
		gFlashMirror.Msg6S = 0x000B;
		gRuntimeSetup.Msg6S = gFlashMirror.Msg6S;	//default CtrlCh
		gFlashMirror.Msg6D = 0x4006;				//data1 =input number
		gRuntimeSetup.Msg6D = 0x4006;				//data2 =0x40
	}
	if ((gFlashMirror.Msg7S & gFlashMirror.Msg7D) == 0xFFFF) {
		gFlashMirror.Msg7S = 0x000B;				//only used when VERO_PCB
		gRuntimeSetup.Msg7S = gFlashMirror.Msg7S;	//default CtrlCh
		gFlashMirror.Msg7D = 0x4007;				//data1 =input number
		gRuntimeSetup.Msg7D = 0x4007;				//data2 =0x40
	}
	if (gFlashMirror.NumBase == 0xFFFF) {
		gFlashMirror.NumBase = 1;				//default hex
		gRuntimeSetup.NumBase = 1;
	}
	gNumBase = gFlashMirror.NumBase;
	gGlobalMidiChannel =  gFlashMirror.Channel;
    gFlashChanged = 0;
    configure_GPIO();
}


void pushFlashPage31(void)   //**********************************************
//save gFlashSave (possibly updated preset) instead of gFlashMirror(operative)
// **************************************************************************
{
uint8_t k;
uint16_t  *flashPtr, *fMirrorPtr;

//unlock flash
#define KEY1    0x45670123      //mandatory unlock key defined by STM
#define KEY2    0xCDEF89AB
    while ((FLASH->SR & FLASH_SR_BSY)!=0); //wait until no operation is going on
    if ((FLASH->CR & FLASH_CR_LOCK) != 0) {
            FLASH->KEYR = KEY1;
            FLASH->KEYR = KEY2;
    }

//erase Page31
    FLASH->CR |= FLASH_CR_PER;
    FLASH->AR = Page31;
    FLASH->CR |= FLASH_CR_STRT;
    while ((FLASH->SR & FLASH_SR_BSY) != 0) ;
    if ((FLASH->SR & FLASH_SR_EOP) !=0)  FLASH->SR |= FLASH_SR_EOP;
    else {
        sendX32(FLASH->SR);        //error management ?
        sendB('!');
        sendB('!');
    }
    FLASH->CR &= ~FLASH_CR_PER;

//write gFlashSave into page31
    flashPtr = (uint16_t *)Page31;
	gFlashMirror.NumBase = (uint16_t)gNumBase;
    fMirrorPtr = (uint16_t *) &(gFlashMirror.ComBaud);
    FLASH->CR |= FLASH_CR_PG;
    for (k=0;k<24;k++)  {
        *flashPtr = *fMirrorPtr;
        while ((FLASH->SR & FLASH_SR_BSY) != 0) ;
        if ((FLASH->SR & FLASH_SR_EOP) != 0)  FLASH->SR |= FLASH_SR_EOP;
        else {
            sendX32(FLASH->SR);        //error management ?
            sendB('!');
            sendB('!');
        }
        flashPtr++;
        fMirrorPtr++;
    }
    FLASH->CR &= ~FLASH_CR_PG;

//lock flash
    FLASH->CR |= FLASH_CR_LOCK;
    msDelay(10);
	gFlashChanged |= 0xFF000000;
}

//###########################################################################

// USART handling
//***************

void initUSART(void) // ****************************************************
// if USB is used with TSSOP package, USART2 is the only available USART
// USART2-TxD is PortA,2
// USART2-RxD is PortA,3
// **************************************************************************
{
uint16_t baudset;

    RCC->AHBENR |= RCC_AHBENR_GPIOAEN ;     // PortA clock enable
    RCC->APB1ENR |=  RCC_APB1ENR_USART2EN ; // USART2 clock enable

    GPIOA->MODER &= ~(GPIO_MODER_MODER2_0 | //Set pin 2,3 to alternate function
                      GPIO_MODER_MODER3_0 ) ;  // 10=Alternate function mode
    GPIOA->MODER |= (GPIO_MODER_MODER2_1 |  //Set pin 2,3 to alternate function
                     GPIO_MODER_MODER3_1 ) ;  // 10=Alternate function mode
    //GPIOA->OTYPER is push-pull by default
    //GPIOA->OSPEEDR is 2 MHz by default
    //GPIOA->PUPDR is HiZ by default (except PA13,14,15)

//USART is AF1
    GPIOA->AFR[0] = 0x00001100;
    GPIOA->AFR[1] = 0x00000000;

    gRXHEAD = gRXBUF;
    gRXTAIL = gRXBUF;
#ifdef USART_TXINTERRUPT
    gTXHEAD = gTXBUF;
    gTXTAIL = gTXBUF;
#endif

    if (guDeviceClass == CDC_CLASS)  {
        switch(gFlashMirror.ComBaud) {
        case 96:
            baudset = 0x1380;       //9600Baud: 312.0 at 48 MHz APB1 Clock
            break;
        case 19:
            baudset = 0x09C0;       //19200Baud: 156.0 at 48 MHz APB1 Clock
            break;
        case 31:
            baudset = 0x0600;       //MIDI=31250Baud: 96.0 at 48 MHz APB1 Clock
            break;
        case 38:
            baudset = 0x04E0 ;      //38400Baud: 78.0 at 48 MHz APB1 Clock
            break;
        case 57:
            baudset = 0x0340;       //57600Baud: 52.0 at 48 MHz APB1 Clock
            break;
        case 11:
            baudset = 0x01A0;       //115200Baud: 26.0 at 48 MHz APB1 Clock
            break;
        default:
            baudset = 0x01A0;       //default (virgin flash)
            break;                  //115200Baud: 26.0 at 48 MHz APB1 Clock;
        }
        USART2->BRR = baudset;		//default 115200Baud
    }
    else {
        switch(gFlashMirror.MidiBaud) {
        case 96:
            baudset = 0x1380;       //9600Baud: 312.0 at 48 MHz APB1 Clock
            break;
        case 19:
            baudset = 0x09C0;       //19200Baud: 156.0 at 48 MHz APB1 Clock
            break;
        case 31:
            baudset = 0x0600;       //MIDI=31250Baud: 96.0 at 48 MHz APB1 Clock
            break;
        case 38:
            baudset = 0x04E0 ;      //38400Baud: 78.0 at 48 MHz APB1 Clock
            break;
        case 57:
            baudset = 0x0340;       //57600Baud: 52.0 at 48 MHz APB1 Clock
            break;
        case 11:
            baudset = 0x01A0;       //115200Baud: 26.0 at 48 MHz APB1 Clock
            break;
        default:
            baudset = 0x0600;		//default (virgin flash)
            break;					//31250Baud: 96.0 at 48 MHz APB1 Clock;
        }
        USART2->BRR = baudset;	 //default:MIDI=31250Baud
    }
#ifdef VERO_PCB
	USART2->CR2 = USART_CR2_TXINV;	//1 Stopbit, TX level inverted
#else
    USART2->CR2 = 0;				//default:1 Stopbit
#endif
    USART2->CR3 = 0;				//default:no Handshake
//changed 01 Dec 2017 / REVNUM 4: serial TX interrupt driven (optionally)
//the TXIE flag will be set when the first byte is going to be transmitted
    USART2->CR1 = USART_CR1_UE | USART_CR1_RXNEIE |
                  USART_CR1_TE | USART_CR1_RE;  //8Bit, no Parity,RX&TX Int ON
    NVIC_EnableIRQ(USART2_IRQn);
}


void sendB(uint8_t data) // *************************************************
//changed 01 Dec 2017 / REVNUM 4: serial TX interrupt driven
// **************************************************************************
{
#ifdef USART_TXINTERRUPT
    *gTXHEAD++ = data;
    if (gTXHEAD == gTXBUFTOP)  gTXHEAD=gTXBUF;
    USART2->CR1 |= USART_CR1_TXEIE;//if new data always renew interrupt enable
#else
    while(!(USART2->ISR & USART_ISR_TXE));
    USART2->TDR = data;
#endif
}


void sendAllB(uint8_t data) // **********************************************
// **************************************************************************
{
    if ((gSetupMode == 0x77) || (gSetupMode == 0)) sendB(data);
    if ((gSetupMode == 0xFF ) || (gSetupMode == 0)) sendUsbByte(data);
}


void sendEncodeB(uint8_t data) // *******************************************
// **************************************************************************
{
    sendB(data);
    enCodeMidi(data);
}

void sendX8(uint8_t data)  // ***********************************************
// **************************************************************************
{
uint8_t temp;

    temp= (data&0xF0) >> 4;
    temp = temp+0x30;
    if (temp>0x39) temp+= 7;
    sendB(temp);
    temp= data&0x0F;
    temp = temp+0x30;
    if (temp>0x39) temp+= 7;
    sendB(temp);
    sendB(0x20);
}


void sendNibble(uint8_t data) // ********************************************
// **************************************************************************

{
uint8_t asciiNibble;

    asciiNibble = data+0x30;
    if (asciiNibble>0x39) asciiNibble+= 7;
    sendB(asciiNibble);
}


void sendAllNibble(uint8_t data) // *****************************************
// **************************************************************************

{
uint8_t asciiNibble;

    asciiNibble = data+0x30;
    if (asciiNibble>0x39) asciiNibble+= 7;
    if ((gSetupMode == 0x77) || (gSetupMode == 0)) sendB(asciiNibble);
    if ((gSetupMode == 0xFF) || (gSetupMode == 0)) sendUsbByte(asciiNibble);
}


void sendAllX8(uint8_t data)  // ********************************************
// **************************************************************************
{
uint8_t temp;

    temp= (data&0xF0) >> 4;
    temp = temp+0x30;
    if (temp>0x39) temp+= 7;
    sendAllB(temp);
    temp= data&0x0F;
    temp = temp+0x30;
    if (temp>0x39) temp+= 7;
    sendAllB(temp);
    sendAllB(0x20);
}


void sendAllX12(uint16_t data) // **********************************************
// send 1 half word in 12 bit hex format (without leading 0x)
// **************************************************************************
{
uint8_t nibble;

	if (data > 0x0FFF)  data = 0x0FFF;		//precaution
    nibble = (uint8_t) ((data>>8) & 0x000F);
    sendAllNibble(nibble);
    nibble = (uint8_t) ((data>>4) & 0x000F);
    sendAllNibble(nibble);
    nibble = (uint8_t) (data & 0x000F);
    sendAllNibble(nibble);
    sendAllB(0x20);
}


void sendAllS12(uint16_t data) // *******************************************
//send signed decimal number string. Leading zeroes are suppressed
//no leading SPACE, but trailing SPACE
//up to  10000
// **************************************************************************
{
uint32_t quotient;

    if (data < 10) goto ONES;
    if (data < 100) goto TENS;
    if (data < 1000) goto HUNDREDS;

    quotient = data/1000;
    data = data - quotient*1000;
    sendAllB((uint8_t)quotient + 0x30);
HUNDREDS:
    quotient = data/100;
    data = data - quotient*100;
    sendAllB((uint8_t)quotient+ 0x30);
TENS:
    quotient = data/10;
    data = data - quotient*10;
    sendAllB((uint8_t)quotient + 0x30);
ONES:
    sendAllB((uint8_t)data + 0x30) ;
    sendAllB(0x20);
}


void sendAllX16(uint16_t data) // *******************************************
// **************************************************************************
{
uint8_t nibble;

    nibble = (uint8_t) ((data>>12) & 0x000F);
    sendAllNibble(nibble);
    nibble = (uint8_t) ((data>>8) & 0x000F);
    sendAllNibble(nibble);
    nibble = (uint8_t) ((data>>4) & 0x000F);
    sendAllNibble(nibble);
    nibble = (uint8_t) (data & 0x000F);
    sendAllNibble(nibble);
    sendAllB(0x20);
}


void sendX32(uint32_t data) // **********************************************
// **************************************************************************
{
uint8_t nibble;

    nibble = (uint8_t) ((data>>28) & 0x0000000F);
    sendNibble(nibble);
    nibble = (uint8_t) ((data>>24) & 0x0000000F);
    sendNibble(nibble);
    nibble = (uint8_t) ((data>>20) & 0x0000000F);
    sendNibble(nibble);
    nibble = (uint8_t) ((data>>16) & 0x0000000F);
    sendNibble(nibble);
    nibble = (uint8_t) ((data>>12) & 0x0000000F);
    sendNibble(nibble);
    nibble = (uint8_t) ((data>>8) & 0x0000000F);
    sendNibble(nibble);
    nibble = (uint8_t) ((data>>4) & 0x0000000F);
    sendNibble(nibble);
    nibble = (uint8_t) (data & 0x0000000F);
    sendNibble(nibble);
    sendB(0x20);
}


void sendAllString(const uint8_t * str) // ***********************************
// ***************************************************************************
{
 uint8_t byte;

    while  ((byte = *str++) != 0)  sendAllB(byte);
}


void sendCRLF(void)  // *****************************************************
// **************************************************************************
{
    sendAllB(0x0D);
    sendAllB(0x0A);
}


uint16_t pullUsartRxBuf(void) // ********************************************
// pull 1 byte asynchronously from USART reveive buffer
// check first if something received. If yes then set bit 15. If not return 0
// less overhead and more safe than use of a separate "buffer filled" flag
// **************************************************************************
{
volatile uint16_t recv;

    if (gSYSCOUNT == 0) servSystem();               //serve watchdog etc.

    if (gRXTAIL != gRXHEAD)  {
        recv= (uint16_t)((*gRXTAIL++ & 0x00FF)|0x8000);
        if (gRXTAIL == gRXBUFTOP)  gRXTAIL = gRXBUF;
        if ((USART2->BRR != 0x0600) || ((recv != 0x80F8)&&(recv != 0x80FE))) {
            //if MIDI baudrate don't blink for Timing Clock or Active Sensing
#ifdef VERO_PCB
			GPIOB->BSRR = GPIO_BSRR_BR_8;		// green LED (PB8) OFF
			gGREEN_LEDCOUNT = 250;				//milliseconds
#else
  #ifdef ALT_LED
			GPIOA->BSRR = GPIO_BSRR_BS_0;		// green LED ON
  #else
			GPIOA->BSRR = GPIO_BSRR_BR_0;		// green LED OFF
  #endif
			gGREEN_LEDCOUNT = 250;				//milliseconds
#endif
        }
        return recv;
    }
    else return 0;
}


uint16_t pullAllRxBuf(void)  // *********************************************
//receive byte from all channels in parallel with priority order USB > RS232
// ********************************************************************
{
volatile uint16_t recv;

   if(((gSetupMode==0xFF)||(gSetupMode==0))&&(recv=pullUsbRxBuf()))return recv;
                                                //USB has priority over USART
 if(((gSetupMode==0x77)||(gSetupMode==0))&&(recv=pullUsartRxBuf()))return recv;
    return 0;
}


#ifdef SETUP_DBG

//buffered least blocking USART Debug Support
// *******************************************

void initSDBG(void) // ******************************************************
// **************************************************************************

{
    gDBGHEAD = gDBGBUF;
}

void sdbgB(uint8_t data) // *************************************************
// **************************************************************************
{
uint32_t k;
const uint8_t overString [] = "--OVERFLOW" ;
	if (gOver == 0)  {
		if (gDBGHEAD < gDBGBUFTOP) *gDBGHEAD++ = data;
		if (gDBGHEAD >= gDBGBUFTOP - 16) {
			k = 0;
			while (overString[k] != 0)  {
				*gDBGHEAD++ = overString[k];
				k++;
			}
			gOver = 1;
		}
	}

}


void sdbgX8(uint8_t data) // ************************************************
// **************************************************************************
{
uint8_t temp;
    temp= (data&0xF0) >> 4;
    temp = temp+0x30;
    if (temp>0x39) temp+= 7;
    sdbgB(temp);
    temp= data&0x0F;
    temp = temp+0x30;
    if (temp>0x39) temp+= 7;
    sdbgB(temp);
    sdbgB(0x20);
}


void sdbgNibble(uint8_t data) // ********************************************
// **************************************************************************

{
uint8_t temp;

    temp = data+0x30;
    if (temp>0x39) temp+= 7;
    sdbgB(temp);
}


void sdbgX16(uint16_t data) // **********************************************
// **************************************************************************
{
uint8_t nibble;

    nibble = (uint8_t) ((data>>12) & 0x000F);
    sdbgNibble(nibble);
    nibble = (uint8_t) ((data>>8) & 0x000F);
    sdbgNibble(nibble);
    nibble = (uint8_t) ((data>>4) & 0x000F);
    sdbgNibble(nibble);
    nibble = (uint8_t) (data & 0x000F);
    sdbgNibble(nibble);
    sdbgB(0x20);
}


void sdbgX32(uint32_t data) // **********************************************
// **************************************************************************
{
uint8_t nibble;

    nibble = (uint8_t) ((data>>28) & 0x0000000F);
    sdbgNibble(nibble);
    nibble = (uint8_t) ((data>>24) & 0x0000000F);
    sdbgNibble(nibble);
    nibble = (uint8_t) ((data>>20) & 0x0000000F);
    sdbgNibble(nibble);
    nibble = (uint8_t) ((data>>16) & 0x0000000F);
    sdbgNibble(nibble);
    nibble = (uint8_t) ((data>>12) & 0x0000000F);
    sdbgNibble(nibble);
    nibble = (uint8_t) ((data>>8) & 0x0000000F);
    sdbgNibble(nibble);
    nibble = (uint8_t) ((data>>4) & 0x0000000F);
    sdbgNibble(nibble);
    nibble = (uint8_t) (data & 0x0000000F);
    sdbgNibble(nibble);
    sdbgB(0x20);

}

void sendDebug(void)  // ****************************************************
// **************************************************************************
{
uint8_t * dbgTail = gDBGBUF;

    sendB(0x0D);
    sendB('D');
    sendB('E');
    sendB('B');
    sendB('U');
    sendB('G');
    sendB(':');
    sendB(0x20);
    while (dbgTail < gDBGHEAD) sendB(*dbgTail++);
    sendB('-');
    sendB('-');
    sendB('E');
    sendB('N');
    sendB('D');
    sendB(0x0D);
    gDBGHEAD = gDBGBUF;
    gOver = 0;
}
#endif


// ##########################################################################

// Interrupt Handlers
//*******************

void SysTick_Handler(void)  // ***********************************************
// ***************************************************************************
{
    if ((gSYSCOUNT > 0)&&(gSYSCOUNT < 0x8000)) gSYSCOUNT--; //controls servSystem()
    else gSYSCOUNT = 0;             //double safetiy against underflow

    if (gGREEN_LEDCOUNT > 0) gGREEN_LEDCOUNT--;
#ifndef VERO_PCB
    if (gRED_LEDCOUNT > 0) gRED_LEDCOUNT--;
#endif
    if (gDELAYCOUNT > 0) gDELAYCOUNT--;

    if (gTIMEOUT > 0) gTIMEOUT--;   //to be used in any subroutine

    if (gGUARDTIME) gGUARDTIME--;
    else {
        if (gStartSetup == 3) {     //device in setup mode
            gStartSetup = 0xFF;
            gUsbStartSetup = 0xFF;
            gSetupMode = 0x77;      //semaphor against competitive access
        }
        else {
            gStartSetup = 0;
        }
    }

    if (gHGUARDTIME) gHGUARDTIME--;
    else {
        if (gUsbStartSetup == 3) {    //host in setup mode
            gStartSetup = 0xFF;
            gUsbStartSetup = 0xFF;
            gSetupMode = 0xFF;      //semaphor against competitive access
        }
        else {
            gUsbStartSetup = 0;
        }
    }
}


void USART2_IRQHandler(void)    // *******************************************
//changed 01 Dec 2017 / REVNUM 4: serial TX interrupt driven
// **************************************************************************
{
    if (USART2->ISR & USART_ISR_RXNE)  {
        *gRXHEAD++ = (uint8_t)(USART2->RDR & 0x00FF);
        if (gRXHEAD==gRXBUFTOP)  gRXHEAD=gRXBUF;
    }
#ifdef USART_TXINTERRUPT
    if (USART2->ISR & USART_ISR_TXE)  {
        if (gTXTAIL == gTXHEAD){                //nothing to transmit
            USART2->CR1 &= ~USART_CR1_TXEIE;    //then switch interrupt off
            return;
        }
        USART2->TDR = *gTXTAIL++ ;
        if (gTXTAIL == gTXBUFTOP)  gTXTAIL=gTXBUF;
    }
#endif
}

// end of file
