// main.h  // ********************************************************
//
// user interface related routines for the project "USB-MidiCom",
// simple data converter for STM32F042F6 (20 pin TSSOP package)
// based on USB MIDISTREAM and CDC-ABSTRACT CONTROL MODEL
//
// Author and (c)2016-21 Wolfgang Schemmert. The copyright claim and
// the GPL-3 Licence is restricted to this file and the depending files
//"main.c", "userinterface.c", "usb-fs-042.c", and "globaldefines.h".
// Other depending material is published and copyrighted
// by STMicroelectronics and Segger GmbH/Rowley Associates Ltd
//
// Contact: www.midi-and-more.de
// A detailled hardware construction and operation manual
// is provided at this website
//
// ****************************************************************************
//      This program is free software: you can redistribute it and/or modify
//      it under the terms of the GNU General Public License as published by
//      the Free Software Foundation, either version 3 of the License, or
//      (at your option) any later version.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//      Licence: GPL-3
//
// ****************************************************************************


// ************************************
//from userinterface.c
extern void userSetup(void);

//from usb-fs.c
extern uint32_t  guDeviceClass, guConfigured;
extern void stopUSB(void);
extern void initUSB(void);
extern void sendUsbByte(uint8_t);
extern uint32_t pullUsbRxBuf(void);
extern void enCodeMidi(uint8_t);
// ************************************

uint8_t gRXBUF[USART_RXBUFLEN];
uint8_t *gRXHEAD=0,*gRXTAIL=0,*gRXBUFTOP=&(gRXBUF[USART_RXBUFLEN]);
#ifdef USART_TXINTERRUPT
uint8_t gTXBUF[USART_TXBUFLEN];
volatile uint8_t *gTXHEAD=0,*gTXTAIL=0,*gTXBUFTOP=&(gTXBUF[USART_TXBUFLEN]);
#endif

//system control
uint32_t gSYSCOUNT=20, gGREEN_LEDCOUNT=0, gRED_LEDCOUNT=0;
uint32_t gDELAYCOUNT=0,gTIMEOUT=0, gGUARDTIME=1000, gHGUARDTIME=1000;
//gHGUARDTIME and gHStartSetup work on USB=Host byte stream
uint32_t gStartSetup, gUsbStartSetup, gSetupMode;
uint32_t gExecOn, gExecBlock, gAdcOn, gFlashChanged, gNumBase;
uint8_t gGlobalMidiChannel;

uint32_t gTest;		//for debugging only
uint32_t gOver;		//exclusively used for SETUP_DBG

#define Page31 (0x08007C00)

struct {        //half words because flash must be programmed in half words
uint16_t ComBaud;
uint16_t MidiBaud;
uint16_t Channel;
uint16_t Hysteresis;
uint16_t VidH;
uint16_t VidL;
uint16_t PidH;
uint16_t PidL;
//user count of inputs starts from '1'
//the offset for a MIDI msg entry relative to "PidH" is = (user I/O num *2)
//new configuration REVNUM 8++: bits 0-7 of MsgXD contain first data byte
//special case=highByte=0x80 or 0x81: send analog input, else digital level
//data2 is stored in gInState[], not in Flash
uint16_t Msg1S;
uint16_t Msg1D;
uint16_t Msg2S;
uint16_t Msg2D;
uint16_t Msg3S;
uint16_t Msg3D;
uint16_t Msg4S;
uint16_t Msg4D;
uint16_t Msg5S;
uint16_t Msg5D;
uint16_t Msg6S;
uint16_t Msg6D;
uint16_t Msg7S;
uint16_t Msg7D;
uint16_t NumBase;	
uint16_t dummy;	//dummy necessary: flash is read in 32 bit words
} gFlashMirror, gRuntimeSetup;	//24 halfwords = 48bytes
//gFlashMirror contains modified setup to be written to Flash
//gRuntimeSetup contains active working setup, possibly changed during session

uint16_t gInTest[IO_NUM +1];    //index 0 is never used
uint16_t gInState[IO_NUM +1];
uint8_t gHysteresis[IO_NUM +1];

const uint8_t gTypeString [] = "USB-MidiCom v" ;
const uint8_t gCopyString [] = " (c)2016-21 W.Schemmert";

void configure_GPIO(void);
void configurePort(uint32_t);
void LED_Off(void);
void msDelay(uint16_t);
void servSystem(void);
void MsgExecutor(void);
void debounceAnalog(uint8_t, uint16_t);
void debounceDigital(uint8_t,uint16_t);
uint16_t readADC(uint8_t);
void initADC(void);
void sendAnaMsg(uint8_t);
void sendDigMsg(uint8_t);
void sendAsciiMsg(uint8_t, uint8_t, uint16_t);
void popFlashPage31(void);
void pushFlashPage31(void);

void initUSART(void);
void sendB(uint8_t);
void sendAllB(uint8_t);
void sendEncodeB(uint8_t);
void sendX8(uint8_t);
void sendAllX8(uint8_t);
void sendNibble(uint8_t);
void sendAllNibble(uint8_t);
void sendAllX12(uint16_t);
void sendAllS12(uint16_t);
void sendX16(uint16_t);
void sendAllX16(uint16_t);
void sendX32(uint32_t);
void sendAllString(const uint8_t *);
void sendCRLF(void);
uint16_t pullUsartRxBuf(void);
uint16_t pullAllRxBuf(void);
void SysTick_Handler(void);
#ifdef MIDICOM_USART1
void USART1_IRQHandler(void);
#else
void USART2_IRQHandler(void);
#endif

#ifdef SETUP_DBG
uint8_t gDBGBUF[4600];
uint8_t *gDBGHEAD=gDBGBUF, *gDBGBUFTOP=&(gDBGBUF[4600]);
void initSDBG(void);
void sdbgB(uint8_t);
void sdbgX8(uint8_t);
void sdbgNibble(uint8_t);
void sdbgX16(uint16_t);
void sdbgX32(uint32_t);
void sendDebug(void);
#endif

